// Procedural brick texture; brick 1
// inspired by ch. 6 in the orange book
// GZ, 2014
// http://shaderfrog.com/app/view/5294

// Set the precision for data types used in this shader
precision mediump float;
precision mediump int;

// Useful parameters:
// BrickStepSize = 0.2 , 0.2
// ambient = 0.4

uniform vec2 BrickStepSize;         // MUST NOT BE 0! use starting value 1.0,0.5

// Default THREE.js uniforms available to both fragment and vertex shader
uniform mat4 modelMatrix;
uniform mat4 modelViewMatrix;
uniform mat4 projectionMatrix;
uniform mat4 viewMatrix;
uniform mat3 normalMatrix;

// Default uniforms provided by ShaderFrog.
uniform vec3 cameraPosition;
uniform float time;

// A uniform unique to this shader. You can modify it using the form
// below the shader preview. Any uniform you add is automatically given a form
uniform vec3 lightPosition;
uniform float ambient;

//  varyings passed from the vertex shader
varying vec3 vPosition_model;     // the position of the vertex/fragment in model(!) coords
varying vec3 vNormal_model;
varying vec3 vPosition_world;
varying vec3 vNormal_world;



void main()
{
    vec3 lightVector = normalize( lightPosition - vPosition_world );

    // An example simple lighting effect, taking the dot product of the normal
    // (which way this pixel is pointing) and a user generated light position
    float brightness = clamp( dot( vNormal_world, lightVector ), 0.0, 1.0);

	vec2 rel_pos = vPosition_model.xy / BrickStepSize;	  // now pos in [k,k+1] for one brick

    // are we in an odd brick row?
    if ( mod(abs(rel_pos.y), 2.0) > 1.0 )     // this saves conversion to int / mod
        rel_pos.x += 0.5;

    rel_pos = fract( rel_pos );                   // = rel. position within brick
    
//  gl_FragColor = vec4(  brightness * vec3(0.8,1.0,0.0), 1.0 );

//  gl_FragColor = vec4( clamp( (brightness+ambient), 0.0, 1.0)  * vec3(rel_pos, 0.0) , 1.0);
    gl_FragColor = vec4( rel_pos, 0.0 , 1.0);   // visualize the rel. position
}

