// Procedural brick texture;
// inspired by ch. 6 in the orange book
// GZ, 2014

// Check Javascript console of browser for error messages during compiling!

// Set the precision for data types used in this shader
precision mediump float;
precision mediump int;

// Default THREE.js uniforms available to both fragment and vertex shader
uniform mat4 modelMatrix;
uniform mat4 modelViewMatrix;
uniform mat4 projectionMatrix;
uniform mat4 viewMatrix;
uniform mat3 normalMatrix;

// Default uniforms provided by ShaderFrog.
uniform vec3 cameraPosition;
uniform float time;

// Default attributes provided by THREE.js. Attributes are only available in the
// vertex shader. You can pass them to the fragment shader using varyings
attribute vec3 position;
attribute vec3 normal;

// Examples of variables passed from vertex to fragment shader
varying vec3 vPosition_model;     // the position of the vertex/fragment in model(!) coords
varying vec3 vNormal_model;
varying vec3 vPosition_world;
varying vec3 vNormal_world;


void main() 
{

    // To pass variables to the fragment shader, you assign them here in the
    // main function. Traditionally you name the varying with vAttributeName
    vNormal_model = normal;
    vPosition_model = position;

    // Calculate the real position of this pixel in 3d space
    vPosition_world = ( modelMatrix * vec4( vPosition_model, 1.0 )).xyz;

    // Calculate the normal including the model rotation and scale
    vNormal_world = normalize( vec3( modelMatrix * vec4( vNormal_model, 0.0 ) ) );

    // This sets the position of the vertex in 3d space. The correct math is
    // provided below to take into account camera and object data.
    gl_Position = projectionMatrix * modelViewMatrix * vec4( position, 1.0 );

}

