// Procedural brick texture;
// inspired by ch. 6 in the orange book
// GZ, 2014
// http://shaderfrog.com/app/view/5295


// Check Javascript console of browser for error messages during compiling!

precision mediump float;
precision mediump int;

// new: ambient light
uniform float ambient;

// Default THREE.js uniforms available to both fragment and vertex shader
uniform mat4 modelMatrix;
uniform mat4 modelViewMatrix;
uniform mat4 projectionMatrix;
uniform mat4 viewMatrix;
uniform mat3 normalMatrix;

// Default uniforms provided by ShaderFrog
uniform vec3 lightPosition;
uniform vec3 cameraPosition;
uniform float time;

// Default attributes provided by THREE.js. Attributes are only available in the
// vertex shader. You can pass them to the fragment shader using varyings
attribute vec3 position;
attribute vec3 normal;

varying vec3 vPosition_model;     // the position of the vertex/fragment in model(!) coords
varying vec3 vNormal_model;
varying vec3 vPosition_world;
varying vec3 vNormal_world;

// new: lighting model in vertex shader
varying float brightness;


void main() 
{
    vNormal_model = normal;
    vPosition_model = position;

    vPosition_world = ( modelMatrix * vec4( vPosition_model, 1.0 )).xyz;
    vNormal_world = normalize( vec3( modelMatrix * vec4( vNormal_model, 0.0 ) ) );

    // new: lighting model (could be implemented more efficiently)
    vec3 lightVector = normalize( lightPosition - vPosition_world );
    float NdotL = max( dot(vNormal_world,lightVector), 0.0 );
	vec3 viewVector = normalize( cameraPosition - vPosition_world );
	vec3 halfVec = normalize( lightVector + viewVector );
    float NdotH = max( dot(vNormal_world,halfVec), 0.0 );
    
    brightness = clamp( NdotL+NdotH+ambient, 0.0, 1.0);

    // This sets the position of the vertex in 3d space. The correct math is
    // provided below to take into account camera and object data.
    gl_Position = projectionMatrix * modelViewMatrix * vec4( position, 1.0 );

}


