#version 330 core

// The "Gooch-Shader", adapted from the orange book
//
// Note: we use a point light source!
//
// GZ, Dec 2013, 2019



// Parameters pre-defined by ShaderMaker (as uniforms)
uniform mat4 viewMatrix;      					// view matrix of the model
uniform mat4 projection;      					// projection matrix (depends on aspect ratio of window)

in vec3 vPosition;
in vec3 vNormal;
out vec3 fNormal;
out vec3 fLightVec;

const vec3 light_pos = vec3(1, 1, 1);         	// we define the position directly in camera coord


void main()
{
	// transform vertex pos into camera space
	vec4 vpos_cam_space = viewMatrix * vec4(vPosition, 1.0);
	gl_Position = projection * vpos_cam_space;

	// compute light vector in camera space
	vec3 vpos3 = vpos_cam_space.xyz / vpos_cam_space.w;
	fLightVec = vec3( light_pos - vpos3 );

	fNormal = ( viewMatrix * vec4(vNormal, 0.0) ).xyz;		// ShaderMaker only uses rigid transforms
}

