// Mandelbrot shader

// Author: Gabriel Zachmann

// Assumption: all texture coords are in [0,1]x[0.1] !

// The vertex shader just scales the texture coords into the 
// range that is specified by the uniforms Min/Max.
// (This *could* be done also in the fragment shader, but doing it here
// distributes the work load a bit better.

// In addition, the vertex shader computes diffuse lighting.


uniform vec2 RangeCenter;			// range in the complex plane
uniform float RangeSize;			// try the following values: center / size
// (see at the bottom of the GUI "glGetUniformf()" )
// 1. -0.6 , 0.0 / 3.0
// 2. -1.63, -0.02 / 0.33
// 3. -1.63 , -0.02 / 0.02
// 4. -1.209677 , 0.236559 / 0.009


void main()
{
	vec2 rangemin = RangeCenter - 0.5*vec2(RangeSize);
	gl_TexCoord[0].st = rangemin + gl_MultiTexCoord0.st * RangeSize;

	// the rest is standard diffuse lighting
	vec3 normal = normalize(gl_NormalMatrix * gl_Normal);
	vec3 lightDir = normalize( gl_LightSource[0].position.xyz );
	float NdotL = max( dot(normal, lightDir), 0.0 );
	vec4 diffuse = gl_FrontMaterial.diffuse * gl_LightSource[0].diffuse;
	vec4 ambient = gl_FrontMaterial.ambient * gl_LightSource[0].ambient;
	vec4 globalAmbient = gl_LightModel.ambient * gl_FrontMaterial.ambient;
	
	gl_FrontColor =  NdotL * diffuse + ambient + globalAmbient;
		
	gl_Position = ftransform();
}
