// Mandelbrot shader
// With coloring using renormalized iteration count.
//
// Author: Gabriel Zachmann, June 2007


#version 110


uniform int MaxIterations;      // max num iterations (try: 10 / 20 / 100 / 700)
uniform vec4 InnerColor;      // two colors that the bands around (try: red)
uniform vec4 OuterColor;      // the mandelbrot set interpolate (try: blue)
uniform float BandFrequ;      // try: 0.1 .. 0.01

vec2 cmul( vec2 c1, vec2 c2 )
{
    return vec2( c1.x * c2.x - c1.y * c2.y ,
                 c1.x * c2.y + c1.y * c2.x );
}


void main()
{
	// iterate function
	vec2 z = vec2( 0.0, 0.0 );
	
	int i;
	for ( i = 0; i < MaxIterations; i ++ )
	{
		z = cmul(z, z) + gl_TexCoord[0].st;
		if ( dot(z,z) > 4.0 )
			break;
	}

	// color according to # iterations
	gl_FragColor = vec4( 0.13, 0.13, 0.13, 1.0 );;
	if ( i < MaxIterations )
	{
		// classical coloring with "banding"
		//float f = float(i) / float(MaxIterations);
		//gl_FragColor = mix( InnerColor, OuterColor, fract(f * BandFrequ) );
		// coloring using 'renormalized iteration count'
		// (see e.g. http://linas.org/art-gallery/escape/smooth.html )
		float f = float(i) + 1.0 - log(log(length(z)));
		gl_FragColor = mix( InnerColor, OuterColor, f * BandFrequ );
	}

	gl_FragColor *= gl_Color;
}
